#include "halview.h"

void Halview::extraSetup()
{
    bitLabels[0] = label_bit_0;
    bitLabels[1] = label_bit_1;
    bitLabels[2] = label_bit_2;
    bitLabels[3] = label_bit_3;
    bitLabels[4] = label_bit_4;
    bitLabels[5] = label_bit_5;
    
    s32Labels[0] = label_s32_0;
    s32Labels[1] = label_s32_1;
    s32Labels[2] = label_s32_2;
    s32Labels[3] = label_s32_3;
    s32Labels[4] = label_s32_4;
    s32Labels[5] = label_s32_5;
    
    floatLabels[0] = label_float_0;
    floatLabels[1] = label_float_1;
    floatLabels[2] = label_float_2;
    floatLabels[3] = label_float_3;            
    floatLabels[4] = label_float_4;
    floatLabels[5] = label_float_5;        
    
    rb_Auto->setChecked(true);
    bAuto = true;
    pb_Refresh->hide();

    struct timeval timeout = { 1, 500000 }; // 1.5 seconds
    context = redisConnectWithTimeout("localhost", 6379, timeout);
    if (context->err) 
        {
        printf("Redis connection error: %s\n", context->errstr);
        qApp->quit();
        }
    
    timer = new QTimer( this );
    
    // signals and slots connections
    connect( pb_Refresh, SIGNAL( clicked() ), this, SLOT( onClickedRefresh() ) );
    connect( rb_Manual, SIGNAL( clicked() ), this, SLOT( onClickedManual() ) );
    connect( rb_Auto, SIGNAL( clicked() ), this, SLOT( onClickedAuto() ) );
    connect( timer, SIGNAL(timeout()), SLOT(onBackgroundRefreshCheck()) );
    connect( pb_FlushDataBase, SIGNAL( clicked() ), this, SLOT( onClickedFlushDataBase() ) );
    
    timer->start( 1000, FALSE );
}

void Halview::onClickedRefresh()
{
// button can't be pressed unless it is active so no need for further checks
    onBackgroundRefresh();
}

void Halview::onClickedManual()
{
    pb_Refresh->show(); 
    bAuto = false;
}

void Halview::onClickedAuto()
{
    pb_Refresh->hide(); 
    bAuto = true;
}

void Halview::onClickedFlushDataBase()
{
    redisCommand(context,"FLUSHDB");
}


// to save a lot of logic testing, this is only way the timer can access the refresh function

void Halview::onBackgroundRefreshCheck()
{
    if(bAuto)
       onBackgroundRefresh(); 
}

void Halview::onBackgroundRefresh()
{
    // code to get values from redis here
int i;
QString str, str1, str2;
void* rep1;    // this is a workaround because g++ will not accept anything other than a void*  to receive redisReply struct
               // even though it then lets you convert using a cast to what it should be afterwards
            
    for (i = 0; i < 6; i ++)
        {
        rep1 = redisCommand(context,"GET B%d", i);
        reply = (redisReply*)rep1;

        str2 = reply->str;
        str1 = (bitLabels[i])->text();
        if(str1 != str2)
            {
            str.sprintf("%d = %s", i, reply->str);
            (bitLabels[i])->setText(str);
            }
        freeReplyObject(reply);
        rep1 = 0L;
        }
        
    for (i = 0; i < 6; i ++)
        {
        rep1 = redisCommand(context,"GET D%d", i);
        reply = (redisReply*)rep1;

        str2 = reply->str;
        str1 = (s32Labels[i])->text();
        if(str1 != str2)
            {
            str.sprintf("%d = %s", i, reply->str);
            (s32Labels[i])->setText(str);
            }
        freeReplyObject(reply);
        rep1 = 0L;
        }
        
    for (i = 0; i < 6; i ++)
        {
        rep1 = redisCommand(context,"GET F%d", i);
        reply = (redisReply*)rep1;

        str2 = reply->str;
        str1 = (floatLabels[i])->text();
        if(str1 != str2)
            {
            str.sprintf("%d = %s", i, reply->str);
            (floatLabels[i])->setText(str);
            }
        freeReplyObject(reply);
        rep1 = 0L;
        }        
        
}


